@@ -35,10 +35,10 @@ import daemon
 import tempfile
 from Crypto import Random
 import paramiko
-from sftpcloudfs.server import CloudFilesSFTPServer
+from sftpcloudfs.server import ObjectStorageSFTPServer
 from sftpcloudfs.constants import version, project_url, config_file, default_ks_service_type, \
     default_ks_tenant_separator, default_ks_endpoint_type
-from ftpcloudfs.fs import CloudFilesFS
+from ftpcloudfs.fs import ObjectStorageFS
 
 class PIDFile(object):
     """
@@ -111,13 +111,15 @@ class Main(object):
                                   'bind-address': "127.0.0.1",
                                   'port': 8022,
                                   'memcache': None,
-                                  'max-children': 20,
+                                  'max-children': "20",
                                   'log-file': None,
                                   'syslog': 'no',
                                   'verbose': 'no',
+                                  'scp-support': 'yes',
                                   'pid-file': None,
                                   'uid': None,
                                   'gid': None,
+                                  'split-large-files': "0",
                                   # keystone auth 2.0 support
                                   'keystone-auth': False,
                                   'keystone-region-name': None,
@@ -134,8 +136,8 @@ class Main(object):
             config.add_section('sftpcloudfs')
 
         parser = OptionParser(version="%prog " + version,
-                              description="This is a SFTP interface to Rackspace " + \
-                                    "Cloud Files and Open Stack Object Storage (Swift).",
+                              description="This is a SFTP interface to OpenStack " + \
+                                    "Object Storage (Swift).",
                               epilog="Contact and support at: %s" % project_url)
 
         parser.add_option("-a", "--auth-url", dest="authurl",
@@ -175,6 +177,11 @@ class Main(object):
                           default=False,
                           help="Run in the foreground (don't detach from terminal)")
 
+        parser.add_option("--disable-scp", dest="no_scp",
+                          action="store_true",
+                          default=not config.getboolean('sftpcloudfs', 'scp-support'),
+                          help="Disable SCP support (default: enabled)")
+
         parser.add_option("--syslog", dest="syslog",
                           action="store_true",
                           default=config.getboolean('sftpcloudfs', 'syslog'),
@@ -207,32 +214,32 @@ class Main(object):
                           action="store_true",
                           dest="keystone",
                           default=config.get('sftpcloudfs', 'keystone-auth'),
-                          help="Use auth 2.0 (Keystone, requires keystoneclient).")
+                          help="Use auth 2.0 (Keystone, requires keystoneclient)")
 
         parser.add_option('--keystone-region-name',
                           type="str",
                           dest="region_name",
                           default=config.get('sftpcloudfs', 'keystone-region-name'),
-                          help="Region name to be used in auth 2.0.")
+                          help="Region name to be used in auth 2.0")
 
         parser.add_option('--keystone-tenant-separator',
                           type="str",
                           dest="tenant_separator",
                           default=config.get('sftpcloudfs', 'keystone-tenant-separator'),
                           help="Character used to separate tenant_name/username in auth 2.0, " + \
-                              "default: TENANT%sUSERNAME." % default_ks_tenant_separator)
+                              "default: TENANT%sUSERNAME" % default_ks_tenant_separator)
 
         parser.add_option('--keystone-service-type',
                           type="str",
                           dest="service_type",
                           default=config.get('sftpcloudfs', 'keystone-service-type'),
-                          help="Service type to be used in auth 2.0, default: %s." % default_ks_service_type)
+                          help="Service type to be used in auth 2.0, default: %s" % default_ks_service_type)
 
         parser.add_option('--keystone-endpoint-type',
                           type="str",
                           dest="endpoint_type",
                           default=config.get('sftpcloudfs', 'keystone-endpoint-type'),
-                          help="Endpoint type to be used in auth 2.0, default: %s." % default_ks_endpoint_type)
+                          help="Endpoint type to be used in auth 2.0, default: %s" % default_ks_endpoint_type)
 
         parser.add_option('--config',
                           type="str",
@@ -258,20 +265,25 @@ class Main(object):
             options.pid_file = "%s/%s.pid" % (tempfile.gettempdir(), __package__)
 
         if options.memcache:
-            CloudFilesFS.memcache_hosts = options.memcache
+            ObjectStorageFS.memcache_hosts = options.memcache
             try:
-                CloudFilesFS(None, None)
+                ObjectStorageFS(None, None, None)
             except (ValueError, TypeError):
                 parser.error("memcache: invalid server address, ip:port expected")
-        else:
-            # default is True, and there's no single worker operation
-            CloudFilesFS.single_cache = False
 
         self.pidfile = PIDFile(options.pid_file)
         if self.pidfile.is_locked():
             parser.error("pid-file found: %s\nIs the server already running?" % options.pid_file)
 
-        options.max_children = config.get('sftpcloudfs', 'max-children')
+        try:
+            options.max_children = int(config.get('sftpcloudfs', 'max-children'))
+        except ValueError:
+            parser.error('max-children: invalid value, integer expected')
+
+        try:
+            options.split_size = int(config.get('sftpcloudfs', 'split-large-files'))*10**6
+        except ValueError:
+            parser.error('split-large-files: invalid size, integer expected')
 
         if options.keystone:
             keystone_keys = ('region_name', 'tenant_separator', 'service_type', 'endpoint_type')
@@ -303,6 +315,8 @@ class Main(object):
             self.log.addHandler(handler)
 
         if self.options.verbose:
+            # enable debug for the root logger (used by ObjectStorageFS)
+            logging.getLogger().setLevel(logging.DEBUG)
             self.log.setLevel(logging.DEBUG)
             self.log.debug(self.options)
         else:
@@ -311,12 +325,14 @@ class Main(object):
 
     def run(self):
         """Run the server."""
-        server = CloudFilesSFTPServer((self.options.bind_address, self.options.port),
-                                       host_key=self.host_key,
-                                       authurl=self.options.authurl,
-                                       max_children=self.options.max_children,
-                                       keystone=self.options.keystone,
-                                       )
+        server = ObjectStorageSFTPServer((self.options.bind_address, self.options.port),
+                                          host_key=self.host_key,
+                                          authurl=self.options.authurl,
+                                          max_children=self.options.max_children,
+                                          keystone=self.options.keystone,
+                                          no_scp=self.options.no_scp,
+                                          split_size=self.options.split_size,
+                                          )
 
         dc = daemon.DaemonContext()
         dc.pidfile = self.pidfile
