@@ -27,33 +27,44 @@ THE SOFTWARE.
 import logging
 
 import os
+import errno
+import shlex
 from SocketServer import StreamRequestHandler, ForkingTCPServer
 
 import paramiko
 from Crypto import Random
 
-from ftpcloudfs.fs import ObjectStorageFS
+from ftpcloudfs.fs import ObjectStorageFS, ObjectStorageFD
 from ftpcloudfs.utils import smart_str
-from functools import wraps
+from sftpcloudfs.scp import SCPHandler
 
+from functools import wraps
 from posixpath import basename
 
 def return_sftp_errors(func):
     """
     Decorator to catch EnvironmentError~s and return SFTP error codes instead.
 
-    Other exceptions are not caught.
+    Other exceptions are logged and processed as EIO errors.
     """
     @wraps(func)
-    def wrapper(*args,**kwargs):
+    def wrapper(*args, **kwargs):
         log = paramiko.util.get_logger("paramiko")
         name = getattr(func, "func_name", "unknown")
         try:
             log.debug("%s(%r,%r): enter" % (name, args, kwargs))
-            rc = func(*args,**kwargs)
-        except EnvironmentError, e:
-            log.debug("%s: caught error: %s" % (name, e))
-            rc = paramiko.SFTPServer.convert_errno(e.errno)
+            rc = func(*args, **kwargs)
+        except BaseException, e:
+            obj = args[0]
+            params = args[1:] if len(args) > 1 else ()
+            msg = "%s%r from %r: %s" % (name, params, obj.client_address, e)
+            if isinstance(e, EnvironmentError):
+                log.info(msg)
+                error = e.errno
+            else:
+                log.exception("unexpected error: %s" % msg)
+                error = errno.EIO
+            rc = paramiko.SFTPServer.convert_errno(error)
         log.debug("%s: returns %r" % (name, rc))
         return rc
     return wrapper
@@ -66,6 +77,7 @@ class SFTPServerInterface(paramiko.SFTPServerInterface):
 
     def __init__(self, server, fs, *args, **kwargs):
         self.fs = fs
+        self.client_address = server.client_address
         self.log = paramiko.util.get_logger("paramiko")
         self.log.debug("%s: start filesystem interface" % self.__class__.__name__)
         super(SFTPServerInterface,self).__init__(server, *args, **kwargs)
@@ -157,6 +169,10 @@ class SFTPHandle(paramiko.SFTPHandle):
         self._file = owner.fs.open(path, mode)
         self._tell = 0
 
+    @property
+    def client_address(self):
+        return self.owner.client_address
+
     @return_sftp_errors
     def close(self):
         self._file.close()
@@ -228,6 +244,7 @@ class ObjectStorageSFTPRequestHandler(StreamRequestHandler):
             self.log.warning("Channel is None, closing")
             t.close()
             return
+
         while t.isAlive():
             t.join(timeout=10)
 
@@ -237,13 +254,15 @@ class ObjectStorageSFTPServer(ForkingTCPServer, paramiko.ServerInterface):
     """
     allow_reuse_address = True
 
-    def __init__(self, address, host_key=None, authurl=None, max_children=20, keystone=None):
+    def __init__(self, address, host_key=None, authurl=None, max_children=20, keystone=None, no_scp=False, split_size=0):
         self.log = paramiko.util.get_logger("paramiko")
         self.log.debug("%s: start server" % self.__class__.__name__)
         self.fs = ObjectStorageFS(None, None, authurl=authurl, keystone=keystone) # unauthorized
         self.host_key = host_key
         self.max_children = max_children
+        self.no_scp = no_scp
         ForkingTCPServer.__init__(self, address, ObjectStorageSFTPRequestHandler)
+        ObjectStorageFD.split_size = split_size
 
     def check_channel_request(self, kind, chanid):
         if kind == 'session':
@@ -255,6 +274,32 @@ class ObjectStorageSFTPServer(ForkingTCPServer, paramiko.ServerInterface):
         # the ObjectStorageSFTPRequestHandler
         return paramiko.OPEN_FAILED_ADMINISTRATIVELY_PROHIBITED
 
+    def check_channel_exec_request(self, channel, command):
+        """Determine if a shell command will be executed for the client."""
+
+        # Parse the command
+        if ' -- ' in command:
+            # scp will use -- to delimit the begining of the unscaped filename
+            # so translate it to something that shelex can manage
+            command = command.replace(' -- ', ' "') + '"'
+        command = shlex.split(command)
+        self.log.debug('check_channel_exec_request %r' % command)
+
+        try:
+            if command[0] == 'scp':
+                if self.no_scp:
+                    self.log.info("scp exec request denied from=%s (scp is disabled)" % (self.client_address,))
+                    return False
+                self.log.info('invoking %r from=%s' % (command, self.client_address))
+                # handle the command execution
+                SCPHandler(command[1:], channel, self.fs, self.log).start()
+                return True
+        except:
+            self.log.exception("command %r failed from=%s" % (command, self.client_address))
+            return False
+
+        return False
+
     def check_auth_none(self, username):
         """Check whether the user can proceed without authentication."""
         return paramiko.AUTH_FAILED
